
#include "GQPE.h"
#include "strutils.h"

#include <stdio.h>
#include <strings.h>

enum {
  kGQPERoundUp          = 1 << 0
};

typedef struct {
  int             peFlags;
  unsigned int    peCPUCount;
} GQPE;

#define CAST_TO_GQPE(P) ((GQPE*)sllNode_GetMutableDataPtr(P))

//

GQPERef
GQPECreate()
{
  GQPERef     newPE = sllNode_CreateWithCapacity(sizeof(GQPE));
  
  if (newPE) {
    GQPE*     data = CAST_TO_GQPE(newPE);
    
    data->peFlags       = 0;
    data->peCPUCount    = 0;
  }
  return newPE;
}

//

GQPERef
GQPECreateWithIDAndCPUCount(
  const char*   peID,
  unsigned int  cpuCount
)
{
  GQPERef     newPE = GQPECreate();
  
  if (newPE) {
    GQPESetIdentifier(newPE,peID);
    CAST_TO_GQPE(newPE)->peCPUCount = cpuCount;
  }
  return newPE;
}

//

GQPERef
GQPERetain(
  GQPERef     aPE
)
{
  return (GQPERef)sllNode_Retain(aPE);
}

//

void
GQPERelease(
  GQPERef     aPE
)
{
  sllNode_Release(aPE);
}

//

const char*
GQPEGetIdentifier(
  GQPERef     aPE
)
{
  return sllNode_GetIdentifier(aPE);
}

//

void
GQPESetIdentifier(
  GQPERef     aPE,
  const char* peID
)
{
  sllNode_SetIdentifier(aPE,peID);
  strtolower((char*)sllNode_GetIdentifier(aPE));
}

//

unsigned int
GQPEGetCPUCount(
  GQPERef     aPE
)
{
  return CAST_TO_GQPE(aPE)->peCPUCount;
}

//

void
GQPESetCPUCount(
  GQPERef       aPE,
  unsigned int  cpuCount
)
{
  CAST_TO_GQPE(aPE)->peCPUCount = cpuCount;
}

//

int
GQPEGetRoundUp(
  GQPERef     aPE
)
{
  return ((CAST_TO_GQPE(aPE)->peFlags & kGQPERoundUp) != 0);
}

//

void
GQPESetRoundUp(
  GQPERef     aPE,
  int         roundUp
)
{
  GQPE*       data = CAST_TO_GQPE(aPE);
  
  if (roundUp)
    data->peFlags |=  kGQPERoundUp;
  else
    data->peFlags &= ~kGQPERoundUp;
}

//

int
GQPEIsValid(
  GQPERef     aPE
)
{
  const char* peID = GQPEGetIdentifier(aPE);
  
  if ((GQPEGetCPUCount(aPE) > 0) && peID && strlen(peID))
    return 1;
  return 0;
}

//

void
GQPESummarize(
  GQPERef     aPE
)
{
  const char* peID = GQPEGetIdentifier(aPE);
  
  if (peID) {
    printf(" %-15s %4d   %s\n",
      peID,
      GQPEGetCPUCount(aPE),
      (GQPEGetRoundUp(aPE) ? "round-up" : "")
    );
  }
}
