//
//  GQParallelEnv
//  Pseudo-classes that implement the parallel-environment selection
//  process.
//
//  Created by Jeffrey Frey on Tue Jan 10 2006.
//  Network & Systems Services
//  University of Delaware
//  Copyright (c) 2006
//

/*
  DTD:
  
    <!ELEMENT pe_lists          (pe_list+)>
    <!ELEMENT pe_list           (pe+)>
    <!ATTLIST pe_list
              id                CDATA   #IMPLIED
              implicit-single   CDATA   #IMPLIED
              script-base       CDATA   #IMPLIED>
              
    <!ELEMENT pe>
    <!ATTLIST pe
              id                CDATA   #IMPLIED
              cpu-count         CDATA   #IMPLIED
              round-up          CDATA   #IMPLIED>
  
  Example:

    <pe_lists>
      <pe_list id="gaussian" script-base="/usr/local/gqueue/scripts">
        <pe id="single" cpu-count="1"/>
        <pe id="dual" cpu-count="2"/>
        <pe id="quad" cpu-count="4" round-up="true"/>
        <pe id="linda" cpu-count="18"/>
      </pe_list>
      <pe_list id="lam-mpi" script-base="/usr/local/gqueue/scripts" implicit-single="yes">
        <pe cpu-count="18"/>
      </pe_list>
    </pe_lists>

*/

#ifndef GQPELIST_H
#define GQPELIST_H

#include "GQPE.h"

typedef sllNodeRef GQPEListRef;

GQPEListRef GQPEListCreate();
GQPEListRef GQPEListCreateWithXML(const char* filename);

GQPEListRef GQPEListRetain(GQPEListRef aPEList);
void GQPEListRelease(GQPEListRef aPEList);

const char* GQPEListGetIdentifier(GQPEListRef aPEList);
void GQPEListSetIdentifier(GQPEListRef aPEList,const char* peListID);

const char* GQPEListGetScriptBase(GQPEListRef aPEList);
void GQPEListSetScriptBase(GQPEListRef aPEList,const char* scriptBase);

enum {
  kGQPEListPreMergeScript,
  kGQPEListValidatePEScript,
  kGQPEListMergeInputScript,
  kGQPEListPostMergeScript
};
char* GQPEListMakeScriptPath(GQPEListRef aPEList,int which);

int GQPEListUseImplicitSingle(GQPEListRef aPEList);

void GQPEListAddPE(GQPEListRef aPEList,GQPERef aPE);
void GQPEListRemovePE(GQPEListRef aPEList,GQPERef aPE);

GQPERef GQPEListSelectPE(GQPEListRef aPEList,unsigned int* nproc);

unsigned int GQPEListMaximumCPUCount(GQPEListRef aPEList);

void GQPEListSummarize(GQPEListRef aPEList);
void GQPEListSummarizeAll(GQPEListRef aPEList);

GQPEListRef GQPEListAppendList(GQPEListRef aPEList,GQPEListRef altPEList);
GQPEListRef GQPEListRemoveList(GQPEListRef aPEList,GQPEListRef targetPEList);
GQPEListRef GQPEListGetListWithIdentifier(GQPEListRef aPEList,const char* peListID);

#endif
