
#include "strutils.h"
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pwd.h>
#include <time.h>
#include <unistd.h>

void
strtolower(
  char*   s
)
{
  while (*s) {
    *s = tolower(*s);
    s++;
  }
}

//

void
strtoupper(
  char*   s
)
{
  while (*s) {
    *s = toupper(*s);
    s++;
  }
}

//

static inline void
fstrnout(
  FILE*     stream,
  char*     s,
  size_t    slen
)
{
  while (slen-- > 0) {
    fputc(*s,stream);
    s++;
  }
}

//

char*
strnchr(
  const char*   s,
  char          c,
  size_t        len
)
{
  const char*   end = s + len;
  
  if (s == NULL)
    return NULL;
  while (*s && *s != c && s < end) s++;
  if (*s == c)
    return (char*)s;
  return NULL;
}

//

char*
strncasestr(
  const char*   big,
  const char*   little,
  size_t        len
)
{
  size_t        bmax,lmax;
  char*         tmp;
  
  if (little == NULL)
    return (char*)big;
  if (*little == '\0')
    return (char*)big;
  if (big == NULL || len == 0)
    return NULL;
  if (*big == '\n')
    return NULL;
    
  lmax = strlen(little);
  tmp = (char*)big;
  bmax = 0;
  while (*tmp) {
    bmax++;
    tmp++;
  }
  
  if (bmax < len)
    len = bmax;
  if (len < lmax)
    return NULL;
  if (lmax == len && toupper(*big) != toupper(*little))
    return NULL;
  len -= lmax;
  
  while (*big && len--) {
    if (toupper(*big) == toupper(*little))
      if (strncasecmp(big,little,lmax) == 0)
        return (char*)big;
    big++;
  }
  return NULL;
}

//

char*
substrn(
  const char*   s,
  size_t        slen,
  size_t        origin,
  size_t        length
)
{
  size_t    idx;
  char*     subs = NULL;
  
  if (origin >= slen)
    return NULL;
  if (origin + length > slen)
    length = slen - origin;
  if (subs = malloc(length + 1)) {
    memcpy(subs,s + origin,length);
    subs[length] = '\0';
  }
  return subs;
}

//

char*
strcatm(
  const char*   s1,...
)
{
  va_list     vargs;
  char*       result = NULL;
  char*       s;
  size_t      idx = 0;
  
  if (s1) {
    result = strdup(s1);
    idx = strlen(result);
    va_start(vargs,s1);
    while (s = va_arg(vargs,char*)) {
      size_t  l = strlen(s);
      
      if (s1 = realloc(result,idx + l + 1)) {
        result = (char*)s1;
        strcpy(result + idx,s);
        idx += l;
      } else {
        free(result);
        result = NULL;
        break;
      }
    }
    va_end(vargs);
  }
  return result;
}

//

char*
strncatm(
  const char*   s1,
  size_t        l1,...
)
{
  va_list     vargs;
  char*       result = NULL;
  char*       s;
  size_t      idx = 0;
  
  if (s1 && l1) {
    result = malloc(l1);memcpy(result,s1,l1);idx = l1;
    va_start(vargs,l1);
    while (s = va_arg(vargs,char*)) {
      if (l1 = va_arg(vargs,size_t)) {
        if (s1 = realloc(result,idx + l1 + 1)) {
          result = (char*)s1;
          strncpy(result + idx,s,l1);
          idx += l1;
        } else {
          free(result);
          result = NULL;
          break;
        }
      }
    }
    va_end(vargs);
  }
  return result;
}

//

#ifndef HAVE_FGETLN

char*
fgetln(
  FILE*   stream,
  size_t* len
)
{
  static char* linebuffer = NULL;
  static size_t blen = 0;
  
  char*         b;
  size_t        l = 0;
  
  if (blen == 0) {
    linebuffer = malloc(1024);
    blen = 1024;
  }
  
  b = linebuffer;
  while (!feof(stream)) {
    if (l == blen) {
      char*   resize = realloc(linebuffer,blen + 64);
      
      if (resize == NULL)
        return NULL;
      blen += 64;
      linebuffer = resize;
      b = linebuffer + l;
    }
    *b = fgetc(stream);
    if (*b == '\n') {
      *len = l + 1;
      return linebuffer;
    } else if  (*b == EOF) {
      *len = l;
      return linebuffer;
    }
    b++;
    l++;
  }
  if (b != linebuffer) {
    *len = l;
    return linebuffer;
  }
  return NULL;
}

#endif

//
#pragma mark -
//

char*
pathstr_pathByDeletingExtension(
  char*   path,
  size_t  plen
)
{
  size_t  idx = 0;
  char*   p = path;
  
  //  Validate the string's length:
  while (idx < plen) {
    if (*p) {
      idx++;
      p++;
    } else
      break;
  }
  //  Backtrack to the '.':
  while (idx > 0) {
    if (*p == '.')
      break;
    else {
      p--;
      idx--;
    }
  }
  //  Make the new string:
  if (idx)
    return substrn(path,idx,0,idx);
  return NULL;
}

//

char*
pathstr_pathByDeletingLastComponent(
  char*   path,
  size_t  plen
)
{
  size_t  idx = 0;
  char*   p = path;
  
  //  Validate the string's length:
  while (idx < plen) {
    if (*p) {
      idx++;
      p++;
    } else
      break;
  }
  //  Backtrack to the '/':
  while (idx > 0) {
    if (*p == '/')
      break;
    else {
      p--;
      idx--;
    }
  }
  //  Make the new string:
  if (idx)
    return substrn(path,idx,0,idx);
  return NULL;
}

//

int
pathstr_fileExists(
  const char* path
)
{
  struct stat   fileopts;
  
  if (stat(path,&fileopts) == 0)
    return 1;
  return 0;
}

//

int
pathstr_isDirectory(
  const char* path
)
{
  struct stat   fileopts;
  
  if (stat(path,&fileopts) == 0)
    return (S_ISDIR(fileopts.st_mode))?(1):(0);
  return 0;
}

//

int
pathstr_isExecutable(
  const char* path
)
{
  uid_t         curUser = getuid();
  gid_t         curGroup = getgid();
  struct stat   fileopts;
  
  if (stat(path,&fileopts) == 0) {
    int         state;
    
    //  Does the current user match the file owner?
    if (fileopts.st_uid == curUser)
      state = fileopts.st_mode & S_IXUSR;
    //  What about matching the group?
    else if (fileopts.st_gid == curGroup)
      state = fileopts.st_mode & S_IXGRP;
    else
      state = fileopts.st_mode & S_IXOTH;
    
    if (state)
      return 1;
  }
  return 0;
}

//

char*
pathstr_basenameForPath(
  const char*   path
)
{
  size_t      start,end;
  char*       s;
  
  end = strlen(path);
  s = (char*)path + end;
  while (end > 0) {
    if (*s == '.')
      break;
    end--;
    s--;
  }
  start = end;
  while (start > 0) {
    if (*s == '/') {
      start++;
      break;
    }
    start--;
    s--;
  }
  if (start != end)
    return substrn(path,strlen(path),start,end - start);
  return NULL;
}

//

char*
pathstr_canonicalPath(
  const char* fname
)
{
  char*       result = NULL;
  
  //  Get rid of whitespace:
  while (*fname && isspace(*fname)) fname++;
  
  switch (*fname) {
  
    case '/': {
      //  If the leading character is a '/' then it's already canonical:
      result = strdup(fname);
      break;
    }
  
    case '~': {
      //  If the leading character is a '~' then do user directory expansion:
      struct passwd*    userrec = NULL;
      size_t            rstart = 2,l1 = strlen(fname);
      
      if (*(fname+1) == '/' || *(fname+1) == '\0') {
        userrec = getpwuid(getuid());
        if (*(fname+1) == '\0')
          rstart = 1;
      } else {
        size_t    end = 0;
        char*     cname = (char*)fname + 1;
        
        while (cname[end] && cname[end] != '/') end++;
        rstart += end;
        {
          char    name[end + 1];
          
          strncpy(name,cname,end);
          name[end] = '\0';
          if ((userrec = getpwnam(name)) == NULL) {
            logtostream(stdout,"unknown user in path: %s",fname);
            return NULL;
          }
        }
      }
      if (userrec) {
        size_t        l2 = strlen(userrec->pw_dir);
        
        l1 -= rstart;
        if (result = malloc(l1 + l2 + 2)) {
          strncpy(result,userrec->pw_dir,l2);
          if (l1) {
            result[l2] = '/';
            strncpy(result+l2+1,fname+rstart,l1);
            result[l1+l2+1] = '\0';
          } else
            result[l2] = '\0';
        }
      } else {
        logtostream(stdout,"weird filesystem error");
        return NULL;
      }
      break;
    }
    
    case '.': {
      //  Relative path, with leading dots:
      char*     workdir = (char*)getcwd(NULL,0);
      size_t    l1 = strlen(workdir),l2;
      char*     fname2 = (char*)fname;
      char*     workdir2 = workdir + l1;
      
      //  Work backwards:  for each '..' delete one component from the
      //  working directory and for each '.' stay where we are:
      while (workdir2 > workdir) {
        if (*fname2 == '.') {
          if (*(fname2+1) == '.') {
            if (*(fname2+2) == '/') {
              fname2 += 3;
              while (*workdir2 != '/' && workdir2 > workdir)
                workdir2--;
              if (workdir2 > workdir)
                workdir2--;
            } else if (*(fname2+2) == '\0') {
              fname2 += 2;
              break;
            } else {
              logtostream(stdout,"input file path is invalid: %s",fname);
              free(workdir);
              return NULL;
            }
          } else if (*(fname2+1) == '\0') {
            break;
          } else if (*(fname2+1) != '/') {
            logtostream(stdout,"input file path is invalid: %s",fname);
            free(workdir);
            return NULL;
          }
        } else
          break;
      }
      
      l2 = strlen(fname2);
      if (workdir2 > workdir) {
        l1 = workdir2 - workdir + 1;
        if (result = malloc(l1 + l2 + 2)) {
          strncpy(result,workdir,l1);
          result[l1] = '/';
          strncpy(result+l1+1,fname2,l2);
          result[l1 + l2 + 1] = '\0';
        }
      } else {
        if (result = malloc(l2 + 2)) {
          result[0] = '/';
          strncpy(result+1,fname,l2);
          result[l2+1] = '\0';
        }
      }
      break;
    }
    
    default: {
      //  Relative path; add the working directory to it:
      char*     workdir = (char*)getcwd(NULL,0);
      size_t    l1 = strlen(workdir),l2 = strlen(fname);
      size_t    slen = l1 + l2 + 2;  //  add the '/' and NUL characters
      
      if (result = realloc(workdir,slen)) {
        result[l1] = '/';
        strncpy(result + l1 + 1,fname,l2);
        result[l1 + l2 + 1] = '\0';
      }
      break;
    }
  }
  return result;
}

//
#pragma mark -
//

void
logtostream(
  FILE*       stream,
  const char* format,
  ...
)
{
  va_list     vargs;
  time_t      tval = time(NULL);
  struct tm*  ltim = localtime(&tval);
  char        tbuf[32];
  
  strftime(tbuf,31,"%b %d %T",ltim);
  
  va_start(vargs,format);
  fprintf(stream,"%s gqueue: ",tbuf);
  vfprintf(stream,format,vargs);
  fputc('\n',stream);
  va_end(vargs);
}
